/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

Zeta Rendering 2007 (Pierluigi Failla) www.zeta.xm.com

TOTAL TIME WORKING ON THIS PROJECT: 166 hours UNTIL 15/04/2007

****************************************************************************/

#include <GL/glew.h>
#include <SDL.h>
//#include <SDL_opengl.h> // commentato senn glew.h scassa!!!

#include <GL/glut.h>
#include <GL/glu.h>

#include <math.h>
#include <time.h>
#include <vector>

#include <vcg/space/point3.h>

// mesh definition
#include <vcg/simplex/vertexplus/base.h>
#include <vcg/simplex/faceplus/base.h>
#include <vcg/complex/trimesh/base.h>
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>
#include <wrap/io_trimesh/import.h>
#include <wrap/gl/trimesh.h>


#include "../client/game_client.h"
#include "game_viewer.h"
#include "race.h"

#include <wrap/gui/trackball.h>

#include "Zeta_Function.h"

#include "tga.h"

#include "ParticleEngine.h"

#include "Lens.h"

/** Client  */
extern  NvmcClient cli;

/** viewport size */
int width=500, height=500;

//terminate program
bool done = false;

// arrows controls
bool c_up, c_down, c_left, c_right;

// vertical rotation angle
float steering = 0.0;
// invisible machine
bool vis = true;

//motion blurring
bool m_b = false; //visualize motion blurring 
bool d_m_b = false; // start stop daemon blurring
//fire on off
bool fire = false;
// lightning data
GLfloat mat_lig[32];
//light0 sun!!!!!!!!!!!
GLfloat lpos[4] = {0,100,0,0};

//cam stuff
float cam_x = 0, cam_y = 0, cam_z = 0;

//start mesh stuff          ply models
using namespace vcg;
using namespace std;
class CEdge;    // dummy prototype never used
class CFace;

class CVertex : public vcg::VertexSimp2< CVertex, CEdge, CFace, vcg::vert::Coord3f, vcg::vert::Normal3f >{};
class CFace   : public vcg::FaceSimp2<   CVertex, CEdge, CFace, vcg::face::VertexRef, vcg::face::Normal3f, face::Color4b > {};
class CMesh   : public vcg::tri::TriMesh< std::vector<CVertex>, std::vector<CFace> > {};

// Globals: the mesh, the OpenGL wrapper to draw the mesh and the trackball.
CMesh mesh_car;
vcg::GlTrimesh<CMesh> _Car;

#define Mesh_Car	"../../client2/mesh/rsq.ply"
//end mesh

//start texture stuff
#define Tex_Street_File			"../../client2/texture/street.bmp" //file name of texture
#define Tex_Start_File			"../../client2/texture/start.bmp" //blocked
#define Tex_World_Bottom_File	"../../client2/texture/grass.bmp" //"../../client2/texture/infiniteplane.bmp"
#define Tex_Pal_File			"../../client2/texture/pal.tga" 
#define Tex_Tree_File			"../../client2/texture/tree.tga"
#define Tex_Build_File			"../../client2/texture/build.bmp"

#define Tex_CubeXpos_File		"../../client2/texture/mount_posx.bmp" //destra
#define Tex_CubeXneg_File		"../../client2/texture/mount_negx.bmp" //sinistra
#define Tex_CubeYpos_File		"../../client2/texture/mount_posy.bmp" //sopra
#define Tex_CubeYneg_File		"../../client2/texture/mount_negy.bmp" //sotto
#define Tex_CubeZpos_File		"../../client2/texture/mount_posz.bmp" //dietro
#define Tex_CubeZneg_File		"../../client2/texture/mount_negz.bmp" //davanti

#define Tex_Flare1_File			"../../client2/texture/Flare1.tga"
#define Tex_Flare2_File			"../../client2/texture/Flare2.tga"
#define Tex_Flare3_File			"../../client2/texture/Flare3.tga"


#define Tex_Street			0			 //index texture
#define Tex_Start			1
#define Tex_World_Bottom	2
#define Tex_Pal				3
#define Tex_Tree			4
#define Tex_Build			5

#define Tex_Skybox_Up		6
#define Tex_Skybox_Down		7
#define Tex_Skybox_Left		8
#define Tex_Skybox_Right	9
#define Tex_Skybox_Front	10
#define Tex_Skybox_Back		11

#define Tex_Flare1			12
#define Tex_Flare2			13
#define Tex_Flare3			14

#define Tex_Particles		15

#define NUM_TEXTURES		16            //total number of texture
//end texture

Particles *P; //particle engine
Flares *Fl; //lens flare

void DynamicCubeMapping(void);

//load all models
void load_models(void){
	printf("loading models...\n");
	//car
	vcg::tri::io::Importer<CMesh>::Open(mesh_car,Mesh_Car);
	// Initialize the mesh itself
	vcg::tri::UpdateBounding<CMesh>::Box(mesh_car);      // update bounding box
	//vcg::tri::UpdateNormals<CMesh>::PerVertexNormalizePerFaceNormalized(mesh); // update Normals
	vcg::tri::UpdateNormals<CMesh>::PerVertexPerFace(mesh_car); // update Normals
	_Car.m = &mesh_car;
	_Car.Update();
}
//texturing
GLuint texName[NUM_TEXTURES];
void init_tex(void){
	printf("init texturing...\n");
	
	bool wrap = true;

	// Generate textures
	glGenTextures(NUM_TEXTURES, texName);
	glPixelStorei(GL_UNPACK_ALIGNMENT,4);
	
	//street
	SDL_Surface* bmpFile = SDL_LoadBMP(Tex_Street_File);

	// Bind texture street texture
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Street] );
	// select modulate to mix texture with color for shading
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	// if wrap is true, the texture wraps over at the edges (repeat)
    //       ... false, the texture ends at the edges (clamp)
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	
	// when texture area is large, bilinear filter the first mipmap
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	// when texture area is small, bilinear filter the closest mipmap
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	
	//free surface
	SDL_FreeSurface(bmpFile);
	
	//World_Bottom grass
	bmpFile = SDL_LoadBMP(Tex_World_Bottom_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_World_Bottom] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	wrap = true;
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	wrap = false;
	SDL_FreeSurface(bmpFile);

	//trees
	tgaInfo *tgaFile;
	tgaFile = tgaLoad(Tex_Tree_File);

	//Bind texture start texture
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Tree] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tgaFile->width, tgaFile->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tgaFile->imageData);
	
	 //paletti
	tgaFile = tgaLoad(Tex_Pal_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Pal] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tgaFile->width, tgaFile->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tgaFile->imageData);

	//palazzi
	bmpFile = SDL_LoadBMP(Tex_Build_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Build] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	wrap = true;
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	wrap = false;
	SDL_FreeSurface(bmpFile);

	//Sky up
	wrap = true;
	bmpFile = SDL_LoadBMP(Tex_CubeYpos_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Skybox_Up] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	SDL_FreeSurface(bmpFile);

	//Sky down
	bmpFile = SDL_LoadBMP(Tex_CubeYneg_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Skybox_Down] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	SDL_FreeSurface(bmpFile);

	//Sky left
	bmpFile = SDL_LoadBMP(Tex_CubeXneg_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Skybox_Left] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	SDL_FreeSurface(bmpFile);

	//Sky right
	bmpFile = SDL_LoadBMP(Tex_CubeXpos_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Skybox_Right] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	SDL_FreeSurface(bmpFile);

	//Sky front
	bmpFile = SDL_LoadBMP(Tex_CubeZneg_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Skybox_Front] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	SDL_FreeSurface(bmpFile);

	//Sky back
	bmpFile = SDL_LoadBMP(Tex_CubeZpos_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Skybox_Back] );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
	wrap = false;
	SDL_FreeSurface(bmpFile);

	//FLARE1
	/*bmpFile = SDL_LoadBMP(Tex_Flare1_File);
	glBindTexture(GL_TEXTURE_2D, texName[Tex_Flare1]);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 1, bmpFile->w, bmpFile->h, GL_LUMINANCE, GL_UNSIGNED_BYTE, bmpFile->pixels);
	SDL_FreeSurface(bmpFile);*/

	tgaFile = tgaLoad(Tex_Flare1_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Flare1] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tgaFile->width, tgaFile->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tgaFile->imageData);

	//FLARE2
	tgaFile = tgaLoad(Tex_Flare2_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Flare2] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tgaFile->width, tgaFile->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tgaFile->imageData);

	//FLARE3
	tgaFile = tgaLoad(Tex_Flare3_File);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_Flare3] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tgaFile->width, tgaFile->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tgaFile->imageData);
}
//cube mapping
void init_cube_mapping(){
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

		/* definiamo il wrapping delle coordinate texture per ogni direzione*/
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);

		/*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e di MINIFICATION */
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

		SDL_Surface* bmpFile;

		bmpFile = SDL_LoadBMP(Tex_CubeXpos_File);
		// Carica la texture per il piano ortogonale a X su x positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(Tex_CubeXneg_File);
		// Carica la texture per il piano ortogonale a X su x negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(Tex_CubeYpos_File);
		// Carica la texture per il piano ortogonale a Y su y positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);	

		bmpFile = SDL_LoadBMP(Tex_CubeYneg_File);
		// Carica la texture per il piano ortogonale a Y su y negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(Tex_CubeZpos_File);
		// Carica la texture per il piano ortogonale a Z su z positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(Tex_CubeZneg_File);
		// Carica la texture per il piano ortogonale a Z su z negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		/*stabilisci come devono essere generate le coordinate texture */
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);

		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}
//cube mapping on
void cube_mapping_on(void){
	/*Abilita la generazione di coordinate texture */
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);  
	/*cube mapping on*/
	glEnable(GL_TEXTURE_CUBE_MAP_EXT);
}
//cube mapping off
void cube_mapping_off(){
   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);
   glDisable(GL_TEXTURE_GEN_R);
   glDisable(GL_TEXTURE_CUBE_MAP_EXT);	
}
//My Car
//int cc=0;
void draw_car(void){
	glPushMatrix();
		glScalef(0.4,0.4,0.4);
		glTranslatef(0,-1.5,0);
	glEnable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	
	float spot1[4]={0.8,0.1,-3.1,1};
    float spot2[4]={-0.8,0.1,-3.1,1};
    float spotd[4]={0,-1,-10,0};

	/*cc = (cc+1)%4;
	glColor3f(1.0,0.5,0.5);
	glPushMatrix();
		glTranslated(cc,cc,cc);
		glutSolidSphere(1,20,20);
	glPopMatrix();*/

    //faro1
    glLightfv(GL_LIGHT1,GL_POSITION, spot1);
	glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION, spotd );

	// faro2
	glLightfv(GL_LIGHT2,GL_POSITION,spot2);
	glLightfv(GL_LIGHT2,GL_SPOT_DIRECTION, spotd );

	if(vis){
		glCullFace(GL_BACK);
		glPushMatrix();
			glScalef(0.007,0.007,0.007);
			glRotatef(90,1,0,0);
			glRotatef(180,0,1,0);
			_Car.Draw<vcg::GLW::DMSmooth,vcg::GLW::CMPerFace,vcg::GLW::TMPerVert> ();
		glPopMatrix();
	}
	float scaling_cyl[16] = {0.4,0,0,0, 0,0.2,0,0, 0,0,0.4,0, 0,0,0,1};
        glColor3f(0.0,0.0,0.0);
        //davanti sinstra
        glPushMatrix();
                glRotatef(90,0,0,1);
                glMultMatrixf(scaling_cyl);
                glTranslated(-1.1,6.5,-4.3);
                glRotatef(steering,1,0,0);
                draw_cylinder();
        glPopMatrix();
        //davanti destra
        glPushMatrix();
                glRotatef(90,0,0,1);
                glMultMatrixf(scaling_cyl);
                glTranslated(-1.1,-6.5,-4.3);
                glRotatef(steering,1,0,0);
                draw_cylinder();
        glPopMatrix();
        //dietro sinistra
        glPushMatrix();
                glRotatef(90,0,0,1);
                glMultMatrixf(scaling_cyl);
                glTranslated(-1.1,6.7,4.7);
                draw_cylinder();
        glPopMatrix();
        //dietro destra
        glPushMatrix();
                glRotatef(90,0,0,1);
                glMultMatrixf(scaling_cyl);
                glTranslated(-1.1,-6.7,4.7);
                draw_cylinder();
        glPopMatrix();
		glPopMatrix();
}
//LIGHTING enabling
void init_lights(void){
	printf("init lighting...\n");
	glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_LIGHT1); //faro 1
        glEnable(GL_LIGHT2); //faro2
    glEnable(GL_NORMALIZE);
}

/** inizializzazione lo stato di opengl */
void init_opengl(void){
	printf("init opengl...\n");
	glPolygonMode(GL_FRONT,GL_FILL); // rasterizza solo i contorni dei poligoni
	int i;
	//MATERIAL
	for(i=0; i < 4; ++i) mat_lig[i]=0.0; //EMISSIVA
	for(i=4; i < 8; ++i) mat_lig[i]=0.4; //AMBIENTE
	for(i=8; i < 12; ++i) mat_lig[i]=0.6; //DIFFUSA
	for(i=12; i < 16; ++i) mat_lig[i]=0.0; //SPECULARE

	for(i=16; i < 20; ++i) mat_lig[i]=0.0; //EMISSIVA
	for(i=20; i < 24; ++i) mat_lig[i]=0.4; //AMBIENTE
	for(i=24; i < 28; ++i) mat_lig[i]=0.6; //DIFFUSA
	for(i=28; i < 32; ++i) mat_lig[i]=0.0; //SPECULARE

	init_lights();
	init_tex();
	init_cube_mapping();
	load_models();
	glEnable(GL_LIGHTING);
	glEnable (GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);
	glShadeModel(GL_SMOOTH);
}

//init Particle Engine
void init_particle_engine(void){
	printf("starting Particle Engine...\n");
	P = new Particles(20000);
	P->SetTexture(Tex_Particles);
	P->InitSystem();
}
//init Flares
float red[3] = {1.0, 0.0, 0.0};
float green[3] = {0.0, 1.0, 0.0};
float blue[3] = {0.0, 0.0, 1.0};
float white[3] = {1,1,1};
void init_flares(void){
	printf("init flares...\n");
	Fl = new Flares(8);
	Fl->F[0].SetFlare(texName[Tex_Flare1],1.5,0.1,red,0.4);
	Fl->F[1].SetFlare(texName[Tex_Flare3],0.8,0.12,white,0.5);
	Fl->F[2].SetFlare(texName[Tex_Flare2],0.4,0.09,white,0.3);
	Fl->F[3].SetFlare(texName[Tex_Flare2],0.7,0.08,white,0.3);
	Fl->F[4].SetFlare(texName[Tex_Flare3],1.1,0.2,blue,0.2);
	Fl->F[5].SetFlare(texName[Tex_Flare1],1.2,0.18,green,0.4);
	Fl->F[6].SetFlare(texName[Tex_Flare2],1.7,0.08,white,0.2);
	Fl->F[7].SetFlare(texName[Tex_Flare2],1.85,0.06,white,0.4);
}
//set light
void set_lights(void){
	printf("setting lights...\n");
	glMaterialfv(GL_FRONT,GL_EMISSION, &mat_lig[0]);
	glMaterialfv(GL_FRONT,GL_AMBIENT, &mat_lig[4]);
	glMaterialfv(GL_FRONT,GL_DIFFUSE, &mat_lig[8]);
	glMaterialfv(GL_FRONT,GL_SPECULAR, &mat_lig[12]);
	glMaterialf(GL_FRONT,GL_SHININESS,100); 
	/* setta i parametri della luce */
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,&mat_lig[16]);
	//ligth0
	glLightfv(GL_LIGHT0,GL_AMBIENT,  &mat_lig[20]);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,  &mat_lig[24]);
	glLightfv(GL_LIGHT0,GL_SPECULAR, &mat_lig[28]);

    //faro 1
	glLightfv(GL_LIGHT1,GL_AMBIENT,  &mat_lig[20]);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,  &mat_lig[24]);
	glLightfv(GL_LIGHT1,GL_SPECULAR, &mat_lig[28]);

	glLighti(GL_LIGHT1,GL_SPOT_EXPONENT, 80 );
	glLightf(GL_LIGHT1,GL_SPOT_CUTOFF, 50 );

    glLightf (GL_LIGHT1, GL_LINEAR_ATTENUATION,    0.0f);
    glLightf (GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.0f);
    glLightf (GL_LIGHT1, GL_CONSTANT_ATTENUATION,  0.0);

    //faro2
    glLightfv(GL_LIGHT2,GL_AMBIENT,  &mat_lig[20]);
	glLightfv(GL_LIGHT2,GL_DIFFUSE,  &mat_lig[24]);
	glLightfv(GL_LIGHT2,GL_SPECULAR, &mat_lig[28]);
        
	glLighti(GL_LIGHT2,GL_SPOT_EXPONENT, 80 );
	glLightf(GL_LIGHT2,GL_SPOT_CUTOFF, 50 );
        
    glLightf (GL_LIGHT2, GL_LINEAR_ATTENUATION,    0.0f);
    glLightf (GL_LIGHT2, GL_QUADRATIC_ATTENUATION, 0.0f);
    glLightf (GL_LIGHT2, GL_CONSTANT_ATTENUATION,  0.0);
}

/** callback function for viewport resize.*/
void resize_screen(int w, int h){
	printf("resizing screen...\n");
	width = w;
	height = h;
	glViewport(0,0,width,height);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);
}


/** draw reference system*/
void draw_axis(void){
	glPushAttrib(0xffffffff);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	glColor3f(1.0,0.0,0.0);
	glVertex3f(0,0,0); //   X
	glVertex3f(10,0,0); 
  
	glColor3f(0.0,1.0,0.0);
	glVertex3f(0,0,0); //   Y
	glVertex3f(0,10,0);
  
	glColor3f(0.0,0.0,1.0);
	glVertex3f(0,0,0); //   Z
	glVertex3f(0,0,10);
	glEnd();
 	glPopAttrib();
	glEnable(GL_LIGHTING);
}


// draw axis
void gl_print(float x, float y, const char *str) {
  glRasterPos2f(x, y);
  int len = (int)strlen(str);
  for(int i = 0; i < len; i++) 
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);
}

//draw external world
void draw_terrain(vcg::Point3f box_min, vcg::Point3f box_max){
	glEnable(GL_LIGHTING);
	glCullFace(GL_FRONT);

	glPushMatrix();
	
	glEnable (GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glBindTexture( GL_TEXTURE_2D, texName[Tex_World_Bottom]);
	glBegin(GL_QUADS); //terrain
		glTexCoord2d(0,0);
		glVertex3f(box_min[0],box_min[1],box_min[2]);
		glTexCoord2d(3000,0);
		glVertex3f(box_max[0],box_min[1],box_min[2]);
		glTexCoord2d(3000,3000);
		glVertex3f(box_max[0],box_min[1],box_max[2]);
		glTexCoord2d(0,3000);
		glVertex3f(box_min[0],box_min[1],box_max[2]);
	glEnd();

	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}
//skybox
void draw_skybox(float x, float y, float z, float width, float height, float length){
	glEnable (GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	// Center the Skybox around the given x,y,z position
	x = x - width  / 2;
	y = y - height / 2;
	z = z - length / 2;

	glPushMatrix();
	// Draw Front side //dietro
	glBindTexture(GL_TEXTURE_2D, texName[Tex_Skybox_Right]);
	glBegin(GL_QUADS);	
		glTexCoord2f(0.99, 0.99); glVertex3f(x,		y,		  z+length);
		glTexCoord2f(0.99, 0.01); glVertex3f(x,		y+height, z+length);
		glTexCoord2f(0.01, 0.01); glVertex3f(x+width, y+height, z+length); 
		glTexCoord2f(0.01, 0.99); glVertex3f(x+width, y,		  z+length);
	glEnd();

	// Draw Back side //davanti
	glBindTexture(GL_TEXTURE_2D, texName[Tex_Skybox_Left]);
	glBegin(GL_QUADS);
		glTexCoord2f(0.99, 0.99); glVertex3f(x+width, y,		  z);
		glTexCoord2f(0.99, 0.01); glVertex3f(x+width, y+height, z); 
		glTexCoord2f(0.01, 0.01); glVertex3f(x,		y+height, z);
		glTexCoord2f(0.01, 0.99); glVertex3f(x,		y,		  z);
	glEnd();

	// Draw Left side //sinsitra
	glBindTexture(GL_TEXTURE_2D, texName[Tex_Skybox_Front]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.99, 0.01); glVertex3f(x,		  y+height,	z);	
		glTexCoord2f(0.01, 0.01); glVertex3f(x,		  y+height,	z+length); 
		glTexCoord2f(0.01, 0.99); glVertex3f(x,		  y,		z+length);
		glTexCoord2f(0.99, 0.99); glVertex3f(x,		  y,		z);		
	glEnd();

	// Draw Right side //destraTex_Skybox_Back
	glBindTexture(GL_TEXTURE_2D, texName[Tex_Skybox_Back]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.01, 0.99); glVertex3f(x+width, y,		z);
		glTexCoord2f(0.99, 0.99); glVertex3f(x+width, y,		z+length);
		glTexCoord2f(0.99, 0.01); glVertex3f(x+width, y+height,	z+length); 
		glTexCoord2f(0.01, 0.01); glVertex3f(x+width, y+height,	z);
	glEnd();

	// Draw Up side
	glBindTexture(GL_TEXTURE_2D, texName[Tex_Skybox_Up]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.01, 0.99); glVertex3f(x+width, y+height, z);
		glTexCoord2f(0.99, 0.99); glVertex3f(x+width, y+height, z+length); 
		glTexCoord2f(0.99, 0.01); glVertex3f(x,		  y+height,	z+length);
		glTexCoord2f(0.01, 0.01); glVertex3f(x,		  y+height,	z);
	glEnd();

	// Draw Down side
	//glBindTexture(GL_TEXTURE_2D, texName[Tex_Skybox_Down]);
	//glBegin(GL_QUADS);		
	//	glTexCoord2f(0.0f, 0.0f); glVertex3f(x,		  y,		z);
	//	glTexCoord2f(1.0f, 0.0f); glVertex3f(x,		  y,		z+length);
	//	glTexCoord2f(1.0f, 1.0f); glVertex3f(x+width, y,		z+length); 
	//	glTexCoord2f(0.0f, 1.0f); glVertex3f(x+width, y,		z);
	//glEnd();

	glPopMatrix();
	glDisable (GL_TEXTURE_2D);
}
//draw scene
void draw_scene(bool dcm){ //dcm if you dont want to reflect something make an if(!dcm){}
	static int cn = 0;
	static int part = 0;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//if(!dcm){
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();//}
	//light on
	set_lights();
	
	//moving sun!!!!
	for(int i=0; i<4; i++){lpos[i] = race.circuit->sun_position[i];}
	
	glLightfv(GL_LIGHT0,GL_POSITION,lpos); //sun!!!!!!!!!!

	/* update the state of the players as received by the server */
	cli.UpdatePlayers();

	Race::PlayersIterator fpi;
	vcg::Point3f position=vcg::Point3f(0,0,0);
	vcg::Point3f delta=vcg::Point3f(0,0,0);
	//if(cli.players.size()>0){
		fpi = cli.players.begin();
		Frame myf=(*fpi).car.frame;
		position=myf.GetRow3(3);
		delta=myf.GetRow3(2)*10+myf.GetRow3(1)*3.5;
	//}
	
	//if(!dcm){
	gluPerspective(75,width/(float)height,1,1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();//}
	vcg::Point3f pov = position + delta*0.4;

	
	//if(!dcm){
		gluLookAt(pov.X()+cam_x,pov.Y()+cam_y,pov.Z()+cam_z,position.X(),position.Y(),position.Z(),0,1,0);
	/*}else{
		gluLookAt(pov.X(),pov.Y(),pov.Z(),position.X(),position.Y(),position.Z(),0,1,0);}*/

	update_eye(pov[0]+cam_x,pov[1]+cam_y,pov[2]+cam_z);

	/* here take the bounding box of the whole circuit */
	vcg::Point3f bmin = race.circuit->bbox.min;
	vcg::Point3f bmax = race.circuit->bbox.max;

	//draw terrain and skybox
	draw_terrain(bmin,bmax);
	glDisable(GL_LIGHTING);
	glDepthMask(false);
	glPushMatrix();
		glTranslatef(0,20,0);
		draw_skybox(position.X(),position.Y(),position.Z(),400,40,400);
	glPopMatrix();
	glDepthMask(true);
	glEnable(GL_LIGHTING);
	
	//axis
	glPushMatrix();
	glScalef(bmax[0]-bmin[0],bmax[1]-bmin[1],bmax[2]-bmin[2]);
	draw_axis();
	glPopMatrix();
	glEnable(GL_LIGHTING);
	
	//circuits
	draw_circuit(texName[Tex_Street],texName[Tex_Pal]);
	//buildings
	draw_buildings(texName[Tex_Build]);
	//tree
	draw_trees(texName[Tex_Tree]);
	
	// draw players
	Race::PlayersIterator pi;
	int st = clock();

	for(pi = cli.players.begin(); pi != cli.players.end(); ++pi){
			glPushMatrix();
				/* apply transformation coded in the frame */
				Frame f = ((*pi).car.frame); 
				Car c = ((*pi).car); 
				steering = c.steering_angle*180/M_PI;
				//cube mapping semplice
				float inv[4][4];
				unsigned int i, j;
				for (i=0; i<4; i++){
					for (j=0; j<4; j++){
						inv[i][j] = f.ElementAt(j,i);}}
				//cube mapping semplice

				glMultMatrixf(&f[0][0]);
				if(fire){P->DrawSystem();}
				if(!dcm){
				glPushMatrix();
					glColor3f(1.0,1.0,1.0);
					glTranslatef(0,1.0,0);
					//cube mapping semplice
					glPushAttrib(GL_TEXTURE_BIT);
					//cube mapping semplice

					cube_mapping_on();

					//cube mapping semplice
					glPushMatrix();					
					glMatrixMode(GL_TEXTURE);
					glPushMatrix();
					glMultMatrixf(&inv[0][0]);  
					glMatrixMode(GL_MODELVIEW);
					//cube mapping semplice

					draw_car();
					//glutSolidCube(1);

					//cube mapping semplice
					glMatrixMode(GL_TEXTURE);
					glPopMatrix();
					glMatrixMode(GL_MODELVIEW);
					glPopMatrix();
					glPopAttrib();
					glFlush();
					//cube mapping semplice

					cube_mapping_off();

				glPopMatrix();
				glDisable(GL_LIGHTING);
				glColor3f(1,1,1);
				gl_print(0.0,0.0,(*pi).ip_address.c_str()); // write the IP address of the client
				glEnable(GL_LIGHTING);
				}
			glPopMatrix();
	}

	if(!dcm){
	vcg::Point3f from(pov[0]+cam_x,pov[1]+cam_y,pov[2]+cam_z);
	vcg::Point3f at(position.X(),position.Y(),position.Z());
	vcg::Point3f pos(lpos[0],lpos[1],lpos[2]);
	Fl->DoFlares(from,at,pos,2);}
	
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glDisable(GL_LIGHTING);
	/* print the commands */
	if(!dcm){
	glColor3f(1,1,1);
	gl_print(-1,0.9,"Exit: ESC");
	gl_print(-1,0.8,"Accel: up");
	gl_print(-1,0.7,"Retro: down");
	gl_print(-1,0.6,"Left:  left");
	gl_print(-1,0.5,"Right: right");
	gl_print(-1,0.4,"Brake: space");
	gl_print(-1,0.3,"Reset: F2");
	gl_print(-1,0.2,"Invis: F1");
	gl_print(-1,0.1,"Cam : (q,a)x; (w,s)y; (e,d)z; c reset");
	gl_print(-1,0.0,"Motion Blurring: Daemon: F10 Blurring Vis: F11");
	gl_print(-1,-0.1,"Fire on-off: f");
	}
	
	glEnable(GL_LIGHTING);

	cn++;
	if(clock()-part >1000){	
		//printf("VIEW fps %d \n\t CAM POS: %f %f %f\n\t steer: %f\n",cn,cam_x,cam_y,cam_z,steering);
		printf("VIEW fps: %d\n",cn);
		part = clock();
		cn=0;
	}
}
// eventi
void keydown(SDL_Event e){
        switch (e.key.keysym.sym){
                case SDLK_UP:
                        c_up = true;
                        break;
                case SDLK_DOWN:
                        c_down = true;
                        break;
                case SDLK_RIGHT:
                        c_right = true;
                        break;
                case SDLK_LEFT:
                        c_left = true;
                        break;
                case SDLK_SPACE:
						cli.Command(COMMAND_BRAKE);
                        break;
                case SDLK_F1:
                        vis =! vis;
                        break;
				case SDLK_F2:
						cli.Command(COMMAND_RESTART);
						break;
				case SDLK_w:
						cam_y++;
						break;
				case SDLK_s:
						cam_y--;
						break;
				case SDLK_q:
						cam_x++;
						break;
				case SDLK_a:
						cam_x--;
						break;
				case SDLK_e:
						cam_z++;
						break;
				case SDLK_d:
						cam_z--;
						break;
				case SDLK_c:
						cam_x = 0;
						cam_y = 0;
						cam_z = 0;
						break;
				case SDLK_F11:
						m_b =! m_b;
						break;
				case SDLK_F10:
						d_m_b =! d_m_b;
						break;
				case SDLK_ESCAPE:
						done = true;
						break;
				case SDLK_f:
						fire =! fire;
						break;
        }
}

void keyup(SDL_Event e){
        switch (e.key.keysym.sym){
                case SDLK_UP:
                        c_up = false;
                        break;
                case SDLK_DOWN:
                        c_down = false;
                        break;
                case SDLK_RIGHT:
                        c_right = false;
                        break;
                case SDLK_LEFT:
                        c_left = false;
                        break;
        }
}
//main!!!!!!!!!!!!!!!!!!!!!!!!!
int launch_viewer(void){
	printf("starting...\n");

	// inizializzazione di SDL  
	SDL_Init(SDL_INIT_VIDEO); 
  
	SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE,16);
    SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE,16);
    SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE,16);
    SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE,16);

	// facciamo una finestra di 500x500
	SDL_SetVideoMode(500,500,32,SDL_OPENGL|SDL_RESIZABLE);//|SDL_NOFRAME);

	// inizializzamo lo stato di opengl
	init_opengl();

	//init Particle System
	init_particle_engine();

	//init Flares
	init_flares();

	// settiamo OpenGL alle dimensioni 600x600
	resize_screen(500,500);
  
	static int last_rendering = 0; //when you have to do rendering
	static int last_blurring = 0; //when you have to blurr
	static int freq = 0;//commands machine independent
	done = false;

	cli.Command(COMMAND_RESTART);

	while (!done) { // loop gestione eventi
		SDL_Event e;
		// guarda se c' un evento. Che ci sia o no continua
		if(SDL_PollEvent(&e))
			// processo l'evento
			switch (e.type) {
				case SDL_KEYDOWN:
					//tasto gi
					keydown(e);
					break;
				case SDL_KEYUP:
					// tasto su
					keyup(e);
					break;
				case SDL_VIDEORESIZE:  
					// evento: la dim della finestra e' cambiata
					resize_screen(e.resize.w,e.resize.h);
					break;
				case SDL_QUIT:  
					// evento: la applicazione deve uscire
					done = true;   
					break;
			}
			//refresh machine independent
			if(clock()-last_rendering>15){
				//temporanemente (???) commentato
				//DynamicCubeMapping();
				draw_scene(false);
				last_rendering = clock();
			}	
			//motion blurring anti phase
			if(clock()-last_blurring>35){
				if(d_m_b){
					printf("Daemon Motion Blurring ON\n");
					motion_blurring_daemon();}
				if(m_b){
					printf("Motion Blurring ON\n");
					motion_blurring();
				}else{
					SDL_GL_SwapBuffers();}
				last_blurring = clock();
			}
			//commands temporization sistemare la sterzata
			if(clock()-freq>20){
				freq = clock();
                if(c_up){
					cli.Command(COMMAND_INCREASE_SPEED);
				}
                if(c_down){
					cli.Command(COMMAND_DECREASE_SPEED);
				}
                if(c_left){
					cli.Command(COMMAND_TURN_LEFT);
				}
				if(c_right){
					cli.Command(COMMAND_TURN_RIGHT);
				}
			}
  }
	return 0;
}

////////////////////////////////////////////Dynamic Cube Mapping//////////////////////////////////////////////

GLuint cube_map_directions[6] = {
	GL_TEXTURE_CUBE_MAP_POSITIVE_X, 
	GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
	GL_TEXTURE_CUBE_MAP_POSITIVE_Y, 
	GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, 
	GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
	GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
};

//Rotate the world to create cube map textures
float CubeMapRots[6][4] = {
	{ -90.0f,	0.0f,	1.0f,	0.0f	},
	{  90.0f,	0.0f,	1.0f,	0.0f	},
	{ -90.0f,	1.0f,	0.0f,	0.0f	},
	{  90.0f,	1.0f,	0.0f,	0.0f	},
	{ 180.0f,	1.0f,	0.0f,	0.0f	},
	{ 180.0f,	0.0f,	0.0f,	1.0f	}
};

GLuint CubeMap;
void DynamicCubeMapping(void){
	bool old_m_b = m_b;
	m_b = false;
	glPushMatrix();
		//Set viewport and fov
		glViewport(0, 0, 512, 512);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(90, 1, 1, 1000);
		glMatrixMode(GL_MODELVIEW);
		
		//SDL_Delay(2000);
		for (int i=0; i<6; i++){
			glClear(GL_DEPTH_BUFFER_BIT);
			glLoadIdentity();

			glRotatef(CubeMapRots[i][0], CubeMapRots[i][1], CubeMapRots[i][2], CubeMapRots[i][3]);

			if (i == 4  ||  i == 5 ){
				glRotatef(180.0, 0.0, 0.0, 1.0);
			}
			glRotatef(-45.0, 0.0, 1.0, 0.0);

			draw_scene(true);

			glEnable(GL_TEXTURE_CUBE_MAP_EXT);
			glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, CubeMap);
			glCopyTexSubImage2D(cube_map_directions[i], 0, 0, 0, 0, 0, 512, 512);
			glFlush();
			glDisable(GL_TEXTURE_CUBE_MAP_EXT);
			
			//per vedere le texture generate decommenta
			//SDL_GL_SwapBuffers(); 
			//SDL_Delay(1500);
		}
	glPopMatrix();
	resize_screen(width,height);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	m_b = old_m_b;
}

///////////////////////////////////////////Speriamo funzioni/////////////////////////////////////////////////