/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.															*
*																			*
* lallopizzi giancarlo														*
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: game_viewer.cpp,v $
Revision 1.23  2007/04/27 10:51:53  glallopizzi
*** empty log message ***

Revision 1.22  2007/04/27 10:10:20  glallopizzi
*** empty log message ***

Revision 1.21  2007/04/26 18:54:54  glallopizzi
*** empty log message ***

Revision 1.20  2007/04/26 12:27:57  glallopizzi
*** empty log message ***

Revision 1.19  2007/04/23 20:52:54  glallopizzi
*** empty log message ***

Revision 1.18  2007/04/23 10:11:07  glallopizzi
*** empty log message ***

Revision 1.17  2007/04/22 09:13:47  glallopizzi
*** empty log message ***

Revision 1.16  2007/04/21 13:19:20  glallopizzi
*** empty log message ***

Revision 1.15  2007/04/20 21:07:45  glallopizzi
*** empty log message ***

Revision 1.14  2007/04/18 11:28:46  glallopizzi
*** empty log message ***

Revision 1.13  2007/04/17 07:55:59  glallopizzi
*** empty log message ***

Revision 1.12  2007/04/14 10:34:46  glallopizzi
*** empty log message ***

Revision 1.11  2007/04/12 21:17:19  glallopizzi
*** empty log message ***

Revision 1.10  2007/04/09 21:31:15  glallopizzi
*** empty log message ***

Revision 1.9  2007/04/07 13:40:39  glallopizzi
*** empty log message ***

Revision 1.8  2007/04/06 11:17:57  glallopizzi
*** empty log message ***

Revision 1.7  2007/04/04 16:21:46  glallopizzi
*** empty log message ***

Revision 1.6  2007/04/04 11:18:02  glallopizzi
*** empty log message ***

Revision 1.5  2007/04/04 11:14:11  glallopizzi
*** empty log message ***

Revision 1.4  2007/04/04 08:39:21  glallopizzi
*** empty log message ***

Revision 1.3  2007/03/30 06:36:48  glallopizzi
*** empty log message ***

Revision 1.2  2007/03/26 07:40:12  glallopizzi
*** empty log message ***

Revision 1.3  2007/03/11 18:11:59  ganovelli
minor modif


****************************************************************************/

#include <SDL.h>
//#include <math.h>
#include <time.h>

#include <vcg/space/point3.h>
#include <vector>

#include <iostream>
#include <GL/glew.h>
#include <GL/glut.h>
#include <GL/glu.h>




#include "../client/game_client.h"

#include "./myFuncs.h"
#include "./myCar.h"
#include "./myTextures.h"
#include "./myLights.h"
#include "./myShadow.h"


#include "game_viewer.h"
#include "race.h"

using std::cout;
using vcg::Point3f;



/** Client  */
extern  NvmcClient cli;

int done=0;
int mb = 0;
float q = 0.91;

/* value of pi */
//#define M_PI       3.14159265358979323846


/** viewport size */
int width,height;

//Guardie per i tasti direzionali
bool static up, down, left, right, brake;

//Guardie per abilitare/disabilitare gli effetti
bool sbox, tball, trs, bdings, tunl, shadow, map, fog, luci, motion_blur, shadowMap;

//Posizione e direzione del mio punto di vista
vcg::Point3f dir, pos, pov;

//La distanza dal centro della macchina al centro del punto di vista
float delta = 3.0;

//Frame della macchina
Frame myf;

// Display lists
GLuint street;
GLuint buildings;
GLuint terrain;
GLuint buildingsShadows;

//Inizializzo lo stato di opengl
void init_opengl(){

	glPolygonMode(GL_FRONT,GL_FILL); 
	glShadeModel(GL_SMOOTH);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	//Depth states
    glClearDepth(1.0f);
    glDepthFunc(GL_LEQUAL);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	glEnable(GL_NORMALIZE);
	
	
	//inizializzo nebbia
	setFog();
	//inizializzo le luci
	initLight();
	//carico le texture
	initTextures();
	//Carico il modello della macchina
	initCar();
	//Calcola la matrice di proiezione delle ombre sul piano
	MakeShadowMatrix();
	
	////////////////////////////////////////
	//Inizializzo e compilo le display list
	street = glGenLists (1);
	glNewList (street, GL_COMPILE);
	drawCircuit();
	glEndList ();

	buildings = glGenLists (2);
	glNewList (buildings, GL_COMPILE);
	drawBuildings(false);
	glEndList ();

	terrain = glGenLists (3);
	glNewList (terrain, GL_COMPILE);
	drawTerrain();
	glEndList ();

	buildingsShadows = glGenLists (4);
	glNewList (buildingsShadows, GL_COMPILE);
	drawBuildings(true);
	glEndList ();
}





void drawAll(bool shadowTexture){
	
	// &&(!shadowTexture) vuol dire che non lo renderizzo quando costruisco la texture di profondit dal punto di vista della luce


	if((sbox)&&(!shadowTexture)){
		//////////////////////////////////////
		//Disegno la skybox intorno alla scena
		//Abilito il cube mapping senza reflection
		glPushMatrix();
		glDisable(GL_CULL_FACE);
		//glEnable(GL_TEXTURE);
		enableCubeMapping(false);
		glDepthMask(false);
		glEnable(GL_LIGHTING);
		drawSkyBox();
		glDisable(GL_LIGHTING);
		glDepthMask(true);
		//glDisable(GL_TEXTURE);
		disableCubeMapping();
		glEnable(GL_CULL_FACE);
		glPopMatrix();
	}	


	///////////////////////////
	//Disegno il terreno
	glPushMatrix();
	switchTexture(3);
	glDisable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glCallList(terrain);
	glEnable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glPopMatrix();


	///////////////////////////
	//Disegno la strada 
	glPushMatrix();
	switchTexture(1);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glCallList(street);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glPopMatrix();

	////////////////////////////
	//Disegno i palazzi
	glPushMatrix();
	switchTexture(2);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
	glCallList(buildings);
	glDisable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();

	if((shadow)&&(!shadowTexture)){
		////////////////////////////
		//Disegno le ombre proiettate degli edifici
		glPushMatrix();
		glDisable(GL_CULL_FACE);
		glCallList(buildingsShadows);
		glEnable(GL_CULL_FACE);
		glPopMatrix();
	}

	
	if((tunl)&&(!shadowTexture)){
		////////////////////////////
		//Disegno i tunnel
		glPushMatrix();
		glDisable(GL_CULL_FACE);
		drawTunnel();
		glEnable(GL_CULL_FACE);
		glPopMatrix();
	}
	
	
	if((trs)&&(!shadowTexture)){
		///////////////////////////
		//Disegno gli alberi
		glPushMatrix();
		switchTexture(4);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glAlphaFunc(GL_GREATER, 0);
		glEnable(GL_ALPHA_TEST);
		glDisable(GL_DEPTH);
		glDisable(GL_CULL_FACE);
		glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
		glDepthMask(false);
		drawTrees();
		glDepthMask(true);
		glEnable(GL_CULL_FACE);
		glEnable(GL_DEPTH);
		glDisable(GL_BLEND);
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_TEXTURE_2D);
		glPopMatrix();
	}
	
		


	////////////////////////////
	// draw players
	Race::PlayersIterator pi;
	
	for(pi = cli.players.begin(); pi != cli.players.end(); ++pi){
		
		glPushMatrix();
			/* apply transformation coded in the frame */
			Frame f = ((*pi).car.frame);
			glMultMatrixf(&f[0][0]);
			glPushMatrix();
				
				glRotatef(-90,1,0,0);
				glRotatef(180,0,0,1);
				glTranslatef(0,0,0.01);
				glPushMatrix();

					//Abilito il cube mapping con reflection
					enableCubeMapping(true);
					glEnable(GL_LIGHTING);
					drawCar(f);
					//draw_axis();
					glDisable(GL_LIGHTING);
					disableCubeMapping();

				glPopMatrix();
				if(shadow){
					glPushMatrix();
						drawShadowCar();
					glPopMatrix();
				}

			glPopMatrix();
			glColor3f(1.0,1.0,1.0);
			gl_print(0.0,0.0,(*pi).ip_address.c_str()); // write the IP address of the client

		glPopMatrix();
	 }

	
	
	
}



//Render the scene
void draw_scene(){
	
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glClearDepth(1.0f);

	// Svuoto il buffer del colore e lo z-buffer
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	// Update the state of the players as received by the server
	cli.UpdatePlayers();

	/////////////////////////////////
	//Prendo il frame della macchina
	Race::PlayersIterator mypi;
	if(!cli.players.empty()){
		mypi=cli.players.begin();
		myf=(*mypi).car.frame;
	}
	//Prendo posizione, direzione e punto di vista
	dir = &myf[2][0];
	pos = &myf[3][0];
	vcg::Point3f pov_aux(pos.X()+delta*dir.X(),pos.Y()+delta*dir.Y(),pos.Z()+delta*dir.Z());
	pov = pov_aux;

	

	// Abilito/Disabilito le luci della macchina
	if(luci)
		carLightEnable();
		//glEnable(GL_LIGHTING);
	else
		carLightDisable();
		//glDisable(GL_LIGHTING);


	// Abilito/Disabilito nebbia
	if(fog){
		glEnable(GL_FOG);
	}else{
		glDisable(GL_FOG);
	}
	
	


	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	float ratio = width/height;
	//Mappa
	if(map)
		glOrtho(-3000,3000,-3000,3000,1,1000);
	else
		gluPerspective(90,ratio,0.1,15000);
	



	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//Mappa
	if(map){
		gluLookAt(0,200,0,0,0,0,0,0,1);
		pos=Point3f(0, 0, 0);
	}
	else
		gluLookAt(pov.X(),pov.Y()+1,pov.Z(),pos.X(),pos.Y(),pos.Z(),0,1,0);
	

	
	// trackball
	if(tball)
		trackApply(map,pos);

	

	if(shadowMap){
		
		//For the first pass, we draw the scene from the light's point of view. 
		//Clear the color and depth buffers and set the matrices to those for the light. 
		//Use a viewport of the same size as the shadow map.

		setMatrixForShadow();

		setDrawForShadow();

		//Draw back faces into the shadow map
		glCullFace(GL_FRONT);
		//Disable color writes, and use flat shading for speed
		glShadeModel(GL_FLAT);
		glColorMask(0, 0, 0, 0);

		//drawAll(true)
		drawAll(shadowMap);

		//Read the depth buffer into texture
		makeShadow();

		//restore states
		glCullFace(GL_BACK);
		glShadeModel(GL_SMOOTH);
		glColorMask(1, 1, 1, 1);
		
		

		glClearColor(0, 0, 0, 1);
		//Pulisco il depth buffer (?e il color buffer?)
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		

	}

	
	
	////////////////////////
	//Rendering della scena
	drawAll(false);	





	//Applico la trackball
	if(tball)
		trackPostDraw();
	

	if(shadowMap){

		shadowApply();

		glEnable(GL_TEXTURE_2D);
		glAlphaFunc(GL_GEQUAL, 0.99f);
		glEnable(GL_ALPHA_TEST);

		shadowTextureCreate();

		drawAll(shadowMap);
		//Disable textures and texgen
		glDisable(GL_TEXTURE_2D);

		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_R);
		glDisable(GL_TEXTURE_GEN_Q);

		//Restore other states
		glDisable(GL_LIGHTING);
		glDisable(GL_ALPHA_TEST);
	}

	////////////////////////
	//Comandi a video

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);

	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	draw_commands();
	

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);

	//SDL_GL_SwapBuffers();
	//cn++;

	


}
//Gestore dell'evento tasto premuto
void keydown(SDL_Event e){
	switch(e.key.keysym.sym){
		//Movimenti
		case SDLK_w: up=true;break;
		case SDLK_s: down=true;break;
		case SDLK_a: left=true;break;
		case SDLK_d: right=true;break;
		case SDLK_x: brake=true;break;
		case SDLK_p: cli.Command(COMMAND_RESTART);break;
		//Effetti
		case SDLK_m: map=!map;break;
		case SDLK_f: fog=!fog;break;
		case SDLK_l: luci=!luci;break;
		case SDLK_r: trackReset();
		case SDLK_F1: sbox=!sbox;break;
		case SDLK_F2: tball=!tball;break;
		case SDLK_F3: trs=!trs;break;
		case SDLK_F4: shadow=!shadow;break;
		case SDLK_F5: tunl=!tunl;break;
		case SDLK_F6: motion_blur=!motion_blur;break;
		case SDLK_F7: shadowMap=!shadowMap;break;
		case SDLK_ESCAPE: done = true;break;
	}
}


//Gestore dell'evento tasto rilasciato
void keyup(SDL_Event e){
	switch(e.key.keysym.sym){
		case SDLK_w: up=false;break;
		case SDLK_s: down=false;break;
		case SDLK_a: left=false;break;
		case SDLK_d: right=false;break;
		case SDLK_x: brake=false;break;
		case SDLK_p: cli.Command(COMMAND_RESTART);break;
		
	}
}



///Lancio il viewer
int launch_viewer(){


  // inizializzazione di SDL  
  SDL_Init( SDL_INIT_VIDEO ); 

  // Dimensioni Accumulation Buffer
  SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE,16);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE,16);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE,16);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE,16);

  
  // facciamo una finestra di 500x500
  SDL_SetVideoMode(1000,800,0, SDL_OPENGL|SDL_RESIZABLE);

	// inizializzamo lo stato di opengl
	init_opengl();

  // settiamo OpenGL alle dimensioni 500x500
  resize_screen(1000,800);

  cli.Command(COMMAND_RESTART);
  
  static int last_rendering = clock();
  static int freq = last_rendering;
  
  while (!done) { // loop gestione eventi
    
    SDL_Event e;
 
		// guarda se c' un evento. Che ci sia o no continua
	if(SDL_PollEvent(&e)){
		// processo l'evento
		switch (e.type) {

			case SDL_KEYDOWN: 
				keydown(e);
				break;

			case SDL_KEYUP:
				keyup(e);
				break;

			case SDL_MOUSEBUTTONDOWN:
			case SDL_MOUSEBUTTONUP:
			case SDL_MOUSEMOTION:
				//Definito in myCar.cpp
				mouse_event(e);
				break;

			case SDL_VIDEORESIZE:  
				// evento: la dim della finestra e' cambiata
				resize_screen(e.resize.w,e.resize.h);
				break;
        
			case SDL_QUIT:  
				// evento: l'applicazione deve uscire
				done=1;   
				break;
			}
		
		
	}

	

	if((!motion_blur)&&(!shadowMap)){

		if( clock() - last_rendering > 20){
			draw_scene();
			SDL_GL_SwapBuffers();
			last_rendering = clock();
		}

	}
	if(shadowMap){
		if( clock() - last_rendering > 35){
			draw_scene();
			SDL_GL_SwapBuffers();
			last_rendering = clock();
		}
	}

	if(motion_blur){

		if( clock() - last_rendering > 10){
			
			switch( mb % 3 ){
				
				case 0:
					q = 0.91;
					draw_scene();
					glAccum(GL_MULT, q);
					glAccum(GL_ACCUM, 1-q);
				break;
				
				case 1:
					q = 0.95;
					draw_scene();
					glAccum(GL_MULT, q);
					glAccum(GL_ACCUM, 1-q);
				break;
				
				case 2:
				q = 0.99;
				draw_scene();
				glAccum(GL_MULT, q);
				glAccum(GL_ACCUM, 1-q);
				glAccum(GL_RETURN, 1.0);
				SDL_GL_SwapBuffers();
				break;

			}
			last_rendering = clock();
			mb++;
		}

	}

	//Invio i comandi al server
	if(clock()-freq > 100){
		if(up) cli.Command(COMMAND_INCREASE_SPEED);
		if(down) cli.Command(COMMAND_DECREASE_SPEED);
		if(left) cli.Command(COMMAND_TURN_LEFT);		    
		if(right) cli.Command(COMMAND_TURN_RIGHT);
		if(brake) cli.Command(COMMAND_BRAKE);
		freq=clock();
	}
  }
	return 0;
}

