/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.															*
*																			*
* lallopizzi giancarlo														*
*                                                                           *
****************************************************************************/

#include <SDL.h>
#include <vcg/space/point3.h>
#include <GL/glut.h>
#include <GL/glu.h>

#include "game_viewer.h"
#include "race.h"
#include "myFuncs.h"




//Valori per il lighting
GLfloat mat_lig[32];

//Posizione del sole
GLfloat sun_pos[4];


void initLight(){

	// Light values and coordinates
	GLfloat ambientLight[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat diffuseLight[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat specularLight[] = { 1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat ambientModel[] = { 0.1f, 0.1f, 0.1f, 0.1f };

	GLfloat ambientLightCar[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat diffuseLightCar[] = { 0.7f, 0.7f, 0.0f, 1.0f };
	GLfloat specularLightCar[] = { 0.7f, 0.7f, 0.0f, 1.0f };

	float mat_ambient[] = { 1.0, 1.0, 1.0, 1.0 };
    float mat_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
    float mat_specular[] = { 0.9, 0.9, 0.9, 1.0 };
    float mat_emission[] = { 0.0, 0.0, 0.0, 1.0 };
    float mat_shininess[] = { 80.0 };


	//Un po' di luce ambiente
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,ambientModel);
	
	//Material
	glMaterialfv(GL_FRONT,GL_EMISSION, mat_emission);
	glMaterialfv(GL_FRONT,GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT,GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT,GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT,GL_SHININESS,mat_shininess);

	//ligth0 - Sole
	glLightfv(GL_LIGHT0,GL_AMBIENT,  ambientLight);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,  diffuseLight);
	glLightfv(GL_LIGHT0,GL_SPECULAR, specularLight);

	//Setto la posizione del sole
	
	sun_pos[0]=race.circuit->sun_position.X();
	sun_pos[1]=race.circuit->sun_position.Y();
	sun_pos[2]=race.circuit->sun_position.Z();
	sun_pos[3]=1.0;

	
	glLightfv(GL_LIGHT0,GL_POSITION,sun_pos);

	//faro 1
	glLightfv(GL_LIGHT1,GL_AMBIENT,  ambientLightCar);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,  diffuseLightCar);
	glLightfv(GL_LIGHT1,GL_SPECULAR, specularLightCar);

	glLighti(GL_LIGHT1,GL_SPOT_EXPONENT, 80 );
	glLightf(GL_LIGHT1,GL_SPOT_CUTOFF, 50 );

    glLightf (GL_LIGHT1, GL_LINEAR_ATTENUATION,    0.001f);
    glLightf (GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.01f);
    glLightf (GL_LIGHT1, GL_CONSTANT_ATTENUATION,  0.1f);

    //faro2
    glLightfv(GL_LIGHT2,GL_AMBIENT,  ambientLightCar);
	glLightfv(GL_LIGHT2,GL_DIFFUSE,  diffuseLightCar);
	glLightfv(GL_LIGHT2,GL_SPECULAR, specularLightCar);
        
	glLighti(GL_LIGHT2,GL_SPOT_EXPONENT, 80 );
	glLightf(GL_LIGHT2,GL_SPOT_CUTOFF, 50 );
        
    glLightf (GL_LIGHT2, GL_LINEAR_ATTENUATION,    0.001f);
    glLightf (GL_LIGHT2, GL_QUADRATIC_ATTENUATION, 0.01f);
    glLightf (GL_LIGHT2, GL_CONSTANT_ATTENUATION,  0.1f);

	
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);

	glEnable(GL_LIGHTING);
}

void carLightEnable(){
	
	//glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);
}

void carLightDisable(){
	
	//glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glDisable(GL_LIGHT2);
}

